/**
 * Imports
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _virtex = require('virtex');

var _setAttribute = require('./setAttribute');

var _setAttribute2 = _interopRequireDefault(_setAttribute);

var _removeAttribute = require('./removeAttribute');

var _removeAttribute2 = _interopRequireDefault(_removeAttribute);

var _createElement = require('./createElement');

var _createElement2 = _interopRequireDefault(_createElement);

/**
 * Vars
 */

var types = _virtex.actions.types;

/**
 * Virtex DOM effects driver
 */

function dom(doc) {
  return function (_ref) {
    var dispatch = _ref.dispatch;
    return function (next) {
      return function (action) {
        return handle(dispatch, next, action);
      };
    };
  };
}

function handle(dispatch, next, action) {
  var type = action.type;
  var payload = action.payload;

  switch (type) {
    case types.CREATE_TEXT_NODE:
      return doc.createTextNode(payload);
    case types.CREATE_ELEMENT:
      return (0, _createElement2['default'])(doc, dispatch, payload.tag, payload.attrs, payload.children);
    case types.SET_ATTRIBUTE:
      return (0, _setAttribute2['default'])(dispatch, payload.node, payload.name, payload.value);
    case types.REMOVE_ATTRIBUTE:
      return (0, _removeAttribute2['default'])(payload.node, payload.name, payload.priorValue);
    case types.APPEND_CHILD:
      return payload.node.appendChild(payload.childNode);
    case types.REPLACE_CHILD:
      return payload.node.replaceChild(payload.newChild, payload.oldChild);
    case types.INSERT_BEFORE:
      return payload.node.insertBefore(payload.newChild, payload.oldChild);
    case types.REMOVE_CHILD:
      return payload.node.removeChild(payload.childNode);
    default:
      return next(action);
  }
}

/**
 * Exports
 */

exports['default'] = dom;
module.exports = exports['default'];